'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _rsocketFlowable = require('rsocket-flowable');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Copyright (c) 2017-present, Netifi Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * 
 */

var MAX_REQUEST_N = 0x7fffffff; // uint31

var SwitchTransformOperator = function () {
  function SwitchTransformOperator(initial, transformer) {
    _classCallCheck(this, SwitchTransformOperator);

    this._transformer = transformer;
    this._initial = initial;
  }

  SwitchTransformOperator.prototype.onComplete = function onComplete() {
    this._actual.onComplete();
  };

  SwitchTransformOperator.prototype.onError = function onError(error) {
    var _this = this;

    if (this._canceled) {
      return;
    }

    if (this._once) {
      this._actual.onError(error);
    } else {
      this._initial.onSubscribe({
        cancel: function cancel() {},
        request: function request() {
          _this._initial.onError(error);
        }
      });
    }
  };

  SwitchTransformOperator.prototype.onNext = function onNext(value) {
    var _this2 = this;

    if (this._canceled) {
      return;
    }

    if (!this._once) {
      this._once = true;
      this._first = value;
      try {
        var result = this._transformer(value, new _rsocketFlowable.Flowable(function (s) {
          return _this2.subscribe(s);
        }));
        result.subscribe(this._initial);
      } catch (e) {
        this.onError(e);
      }
      return;
    }

    this._actual.onNext(value);
  };

  SwitchTransformOperator.prototype.onSubscribe = function onSubscribe(subscription) {
    this._subscription = subscription;
  };

  SwitchTransformOperator.prototype.request = function request(n) {
    if (this._first) {
      var f = this._first;
      this._first = undefined;
      this._actual.onNext(f);

      if (MAX_REQUEST_N <= n) {
        this._subscription.request(MAX_REQUEST_N);
      } else if (--n > 0) {
        this._subscription.request(n);
      }
    } else {
      this._subscription.request(n);
    }
  };

  SwitchTransformOperator.prototype.cancel = function cancel() {
    this._canceled = true;
    this._first = undefined;
  };

  SwitchTransformOperator.prototype.subscribe = function subscribe(actual) {
    if (actual && !this._actual) {
      this._actual = actual;
      this._actual.onSubscribe(this);
    } else if (actual) {
      if (actual.onError) {
        actual.onError(new Error('SwitchTransform allows only one Subscriber'));
      }
    }
  };

  SwitchTransformOperator.prototype.map = function map(fn) {
    var _this3 = this;

    return new _rsocketFlowable.Flowable(function (subscriber) {
      return _this3.subscribe(subscriber);
    }).map(fn);
  };

  return SwitchTransformOperator;
}();

exports.default = SwitchTransformOperator;